'''Graphical routines for the game.'''

# Pygame libs #
import pygame

# Game libs #
import data

class Animation(object):
    def __init__(self, frame_ids, frame_times):
        self.timer = 0
        self.frame_ids   = frame_ids
        self.frame_times = frame_times
        self.total_time  = sum(frame_times)

    def reset(self):
        self.timer = 0
    
    def update(self, d_time):
        self.timer = (self.timer + d_time) % self.total_time
    
    def get_frame_id(self):
        time_left = self.timer - self.frame_times[0]
        index     = 0
        while time_left > 0:
            time_left -= self.frame_times[index]
            index     += 1
        return self.frame_ids[index]

class Sprite(object):
    def __init__(self, spritesheet):
        self.spritesheet       = spritesheet
        self.current_animation = None
        self.animations        = {}
    
    def add_animation(self, name, animation):
        self.animations[name] = animation
    
    def draw(self, dest_surface, dest_position):
        frame_id = self.current_animation.get_frame_id()
        self.spritesheet.draw(frame_id, dest_surface, dest_position)

    def get_frame(self):
        frame_id = self.current_animation.get_frame_id()
        return self.spritesheet.get_frame(frame_id)

    def set_animation(self, name):
        new_animation = self.animations[name]
        if new_animation != self.current_animation:
            self.current_animation = new_animation
            self.current_animation.reset()
        
    def update(self, d_time):
        self.current_animation.update(d_time)

class SpriteSheet(object):
    def __init__(self, image_name, (width, height)):
        '''Loads an image and creates a spritesheet of frames of a given size.'''
        self.width   = width
        self.height  = height
        self.name    = image_name
        self.surface = data.load_image_transparency(image_name)
        
        # ERASE THIS #
        self.surface.set_alpha(128)
        
        # Get frames #
        self.frames  = []
        cols = self.surface.get_width()  // width
        rows = self.surface.get_height() // height
        for i in xrange(rows*cols):
            x = (i %  cols) * width
            y = (i // cols) * height
            area = pygame.rect.Rect(x, y, width, height)
            self.frames.append(self.surface.subsurface(area))
    
    def draw(self, frame_id, dest_surface, dest_position):
        '''Draws the frame at the given surface and at the given location.'''
        dest_surface.blit(self.frames[frame_id], dest_position)
    
    def get_frame(self, frame_id):
        return self.frames[frame_id]

    def __str__(self):
        return "<SpriteSheet Object - Name: {0} - Frame size: ({1}, {2}), Frames: {3}>".format(self.name, self.width, self.height, len(self.frames))
